@@ -11,7 +11,7 @@
 
 #ifdef OPENSSL_SYS_VMS
   /* So fd_set and friends get properly defined on OpenVMS */
-# define _XOPEN_SOURCE_EXTENDED 1
+# define _XOPEN_SOURCE_EXTENDED
 #endif
 
 #include <stdio.h>
@@ -33,12 +33,6 @@
 #include <openssl/bn.h>
 #include <openssl/x509v3.h>
 
-#if defined(__TANDEM)
-# if defined(OPENSSL_TANDEM_FLOSS)
-#  include <floss.h(floss_fork)>
-# endif
-#endif
-
 #if defined(OPENSSL_SYS_VXWORKS)
 /* not supported */
 int setpgid(pid_t pid, pid_t pgid)
@@ -672,8 +666,7 @@ redo_accept:
                 resp =
                     OCSP_response_create(OCSP_RESPONSE_STATUS_MALFORMEDREQUEST,
                                          NULL);
-                if (resp != NULL)
-                    send_ocsp_response(cbio, resp);
+                send_ocsp_response(cbio, resp);
             }
             goto done_resp;
         }
@@ -771,19 +764,17 @@ redo_accept:
         BIO_free(derbio);
     }
 
-    if (resp != NULL) {
-        i = OCSP_response_status(resp);
-        if (i != OCSP_RESPONSE_STATUS_SUCCESSFUL) {
-            BIO_printf(out, "Responder Error: %s (%d)\n",
-                       OCSP_response_status_str(i), i);
-            if (!ignore_err)
+    i = OCSP_response_status(resp);
+    if (i != OCSP_RESPONSE_STATUS_SUCCESSFUL) {
+        BIO_printf(out, "Responder Error: %s (%d)\n",
+                   OCSP_response_status_str(i), i);
+        if (!ignore_err)
                 goto end;
-        }
-
-        if (resp_text)
-            OCSP_RESPONSE_print(out, resp, 0);
     }
 
+    if (resp_text)
+        OCSP_RESPONSE_print(out, resp, 0);
+
     /* If running as responder don't verify our own response */
     if (cbio != NULL) {
         /* If not unlimited, see if we took all we should. */
