@@ -17,6 +17,7 @@ use TLSProxy::Record;
 use TLSProxy::Message;
 use TLSProxy::ClientHello;
 use TLSProxy::ServerHello;
+use TLSProxy::HelloVerifyRequest;
 use TLSProxy::EncryptedExtensions;
 use TLSProxy::Certificate;
 use TLSProxy::CertificateRequest;
@@ -76,30 +77,33 @@ BEGIN
 my $is_tls13 = 0;
 my $ciphersuite = undef;
 
-sub new
-{
-    my $useSockInet = 0;
-    eval {
-        require IO::Socket::IP;
-        my $s = IO::Socket::IP->new(
-                LocalAddr => "::1",
-                LocalPort => 0,
-                Listen=>1,
-                );
-            $s or die "\n";
-            $s->close();
-    };
-    if ($@ eq "") {
-        require IO::Socket::IP;
-    } else {
-        $useSockInet = 1;
-    }
-
+sub new {
     my $class = shift;
     my ($filter,
         $execute,
         $cert,
         $debug) = @_;
+    return init($class, $filter, $execute, $cert, $debug, 0);
+}
+
+sub new_dtls {
+    my $class = shift;
+    my ($filter,
+        $execute,
+        $cert,
+        $debug) = @_;
+    return init($class, $filter, $execute, $cert, $debug, 1);
+}
+
+sub init
+{
+    require IO::Socket::IP;
+    my $class = shift;
+    my ($filter,
+        $execute,
+        $cert,
+        $debug,
+        $isdtls) = @_;
 
     my $test_client_port;
 
@@ -114,13 +118,8 @@ sub new
         $test_client_port = 49152 + int(rand(65535 - 49152));
         my $test_sock;
         if ($useINET6 == 0) {
-            if ($useSockInet == 0) {
-                $test_sock = IO::Socket::IP->new(LocalPort => $test_client_port,
-                                                 LocalAddr => $test_client_addr);
-            } else {
-                $test_sock = IO::Socket::INET->new(LocalAddr => $test_client_addr,
-                                                   LocalPort => $test_client_port);
-            }
+            $test_sock = IO::Socket::IP->new(LocalPort => $test_client_port,
+                                             LocalAddr => $test_client_addr);
         } else {
             $test_sock = IO::Socket::INET6->new(LocalAddr => $test_client_addr,
                                                 LocalPort => $test_client_port,
@@ -151,6 +150,7 @@ sub new
         sessionfile => undef,
 
         #Public read
+        isdtls => $isdtls,
         proxy_port => 0,
         client_port => $test_client_port,
         server_port => 0,
@@ -170,29 +170,6 @@ sub new
         message_list => [],
     };
 
-    # Create the Proxy socket
-    my $proxaddr = $self->{proxy_addr};
-    $proxaddr =~ s/[\[\]]//g; # Remove [ and ]
-    my @proxyargs = (
-        LocalHost   => $proxaddr,
-        LocalPort   => 0,
-        Proto       => "tcp",
-        Listen      => SOMAXCONN,
-       );
-
-    if (my $sock = $IP_factory->(@proxyargs)) {
-        $self->{proxy_sock} = $sock;
-        $self->{proxy_port} = $sock->sockport();
-        $self->{proxy_addr} = $sock->sockhost();
-        $self->{proxy_addr} =~ s/(.*:.*)/[$1]/;
-        print "Proxy started on port ",
-              "$self->{proxy_addr}:$self->{proxy_port}\n";
-        # use same address for s_server
-        $self->{server_addr} = $self->{proxy_addr};
-    } else {
-        warn "Failed creating proxy socket (".$proxaddr.",0): $!\n";
-    }
-
     return bless $self, $class;
 }
 
@@ -262,7 +239,7 @@ sub connect_to_server
 
     my $sock = $IP_factory->(PeerAddr => $servaddr,
                              PeerPort => $self->{server_port},
-                             Proto => 'tcp');
+                             Proto => $self->{isdtls} ? 'udp' : 'tcp');
     if (!defined($sock)) {
         my $err = $!;
         kill(3, $self->{real_serverpid});
@@ -277,12 +254,50 @@ sub start
     my ($self) = shift;
     my $pid;
 
+    # Create the Proxy socket
+    my $proxaddr = $self->{proxy_addr};
+    $proxaddr =~ s/[\[\]]//g; # Remove [ and ]
+    my $clientaddr = $self->{client_addr};
+    $clientaddr =~ s/[\[\]]//g; # Remove [ and ]
+
+    my @proxyargs;
+
+    if ($self->{isdtls}) {
+        @proxyargs = (
+            LocalHost   => $proxaddr,
+            LocalPort   => 0,
+            PeerHost   => $clientaddr,
+            PeerPort   => $self->{client_port},
+            Proto       => "udp",
+        );
+    } else {
+        @proxyargs = (
+            LocalHost   => $proxaddr,
+            LocalPort   => 0,
+            Proto       => "tcp",
+            Listen      => SOMAXCONN,
+        );
+    }
+
+    if (my $sock = $IP_factory->(@proxyargs)) {
+        $self->{proxy_sock} = $sock;
+        $self->{proxy_port} = $sock->sockport();
+        $self->{proxy_addr} = $sock->sockhost();
+        $self->{proxy_addr} =~ s/(.*:.*)/[$1]/;
+        print "Proxy started on port ",
+            "$self->{proxy_addr}:$self->{proxy_port}\n";
+        # use same address for s_server
+        $self->{server_addr} = $self->{proxy_addr};
+    } else {
+        warn "Failed creating proxy socket (".$proxaddr.",0): $!\n";
+    }
+
     if ($self->{proxy_sock} == 0) {
         return 0;
     }
 
     my $execcmd = $self->execute
-        ." s_server -max_protocol TLSv1.3 -no_comp -rev -engine ossltest"
+        ." s_server -no_comp -engine ossltest -state"
         #In TLSv1.3 we issue two session tickets. The default session id
         #callback gets confused because the ossltest engine causes the same
         #session id to be created twice due to the changed random number
@@ -292,6 +307,14 @@ sub start
         ." -accept $self->{server_addr}:0"
         ." -cert ".$self->cert." -cert2 ".$self->cert
         ." -naccept ".$self->serverconnects;
+    if ($self->{isdtls}) {
+        $execcmd .= " -dtls -max_protocol DTLSv1.2"
+                    # TLSProxy does not support message fragmentation. So
+                    # set a high mtu and fingers crossed.
+                    ." -mtu 1500";
+    } else {
+        $execcmd .= " -rev -max_protocol TLSv1.3";
+    }
     if ($self->ciphers ne "") {
         $execcmd .= " -cipher ".$self->ciphers;
     }
@@ -308,6 +331,7 @@ sub start
     open(my $savedin, "<&STDIN");
 
     # Temporarily replace STDIN so that sink process can inherit it...
+    open(STDIN, "$^X -e 'sleep(10)' |") if $self->{isdtls};
     $pid = open(STDIN, "$execcmd 2>&1 |") or die "Failed to $execcmd: $!\n";
     $self->{real_serverpid} = $pid;
 
@@ -373,11 +397,24 @@ sub clientstart
 {
     my ($self) = shift;
 
+    my $succes = 1;
+
     if ($self->execute) {
         my $pid;
         my $execcmd = $self->execute
-             ." s_client -max_protocol TLSv1.3 -engine ossltest"
+             ." s_client -engine ossltest"
              ." -connect $self->{proxy_addr}:$self->{proxy_port}";
+        if ($self->{isdtls}) {
+            $execcmd .= " -dtls -max_protocol DTLSv1.2"
+                        # TLSProxy does not support message fragmentation. So
+                        # set a high mtu and fingers crossed.
+                        ." -mtu 1500"
+                        # UDP has no "accept" for sockets which means we need to
+                        # know were to send data back to.
+                        ." -bind $self->{client_addr}:$self->{client_port}";
+        } else {
+            $execcmd .= " -max_protocol TLSv1.3";
+        }
         if ($self->cipherc ne "") {
             $execcmd .= " -cipher ".$self->cipherc;
         }
@@ -424,7 +461,9 @@ sub clientstart
     }
 
     my $client_sock;
-    if(!($client_sock = $self->{proxy_sock}->accept())) {
+    if($self->{isdtls}) {
+        $client_sock = $self->{proxy_sock}
+    } elsif (!($client_sock = $self->{proxy_sock}->accept())) {
         warn "Failed accepting incoming connection: $!\n";
         return 0;
     }
@@ -448,6 +487,9 @@ sub clientstart
                     && $self->{saw_session_ticket};
         }
         if (!(@ready = $fdset->can_read(1))) {
+            last if TLSProxy::Message->success()
+                && $self->{saw_session_ticket};
+
             $ctr++;
             next;
         }
@@ -481,7 +523,8 @@ sub clientstart
 
     if ($ctr >= 10) {
         kill(3, $self->{real_serverpid});
-        die "No progress made";
+        print "No progress made\n";
+        $succes = 0;
     }
 
     END:
@@ -522,7 +565,7 @@ sub clientstart
     print "Waiting for s_client process to close: $pid...\n";
     waitpid($pid, 0);
 
-    return 1;
+    return $succes;
 }
 
 sub process_packet
@@ -550,7 +593,9 @@ sub process_packet
     #Return contains the list of record found in the packet followed by the
     #list of messages in those records and any partial message
     my @ret = TLSProxy::Record->get_records($server, $self->flight,
-                                            $self->{partial}[$server].$packet);
+                                            $self->{partial}[$server].$packet,
+                                            $self->{isdtls});
+
     $self->{partial}[$server] = $ret[2];
     push @{$self->{record_list}}, @{$ret[0]};
     push @{$self->{message_list}}, @{$ret[1]};
@@ -788,4 +833,10 @@ sub ciphersuite
     return $ciphersuite;
 }
 
+sub isdtls
+{
+    my $self = shift;
+    return $self->{isdtls}; #read-only
+}
+
 1;
